/*
  Thingspeak and MQTT 
  Meassure Clima Data
  Change SSID and PASSWORD.
  Change Thingsspeak Access Data
*/

#include <NanoESP.h>
#include <NanoESP_MQTT.h>
#include <SoftwareSerial.h>

#define SSID "[Your SSID]"
#define PASSWORD "[Your Password]"

#define THINGSPEAK_CHANNEL_ID "842535"
#define THINGSPEAK_WRITE_API_KEY "J88WXG8DF1JID9TB"
#define THINGSPEAK_MQTT_API_KEY "YNH3T1JOIZ9MM9AI"

#define LED_WLAN 13

#define TEMP A7
#define LIGHT A6

NanoESP nanoesp = NanoESP();
NanoESP_MQTT mqtt = NanoESP_MQTT(nanoesp);

//topic,         value,    QOS, retain
mqtt_msg newValues = {"channels/" + String(THINGSPEAK_CHANNEL_ID) + "/publish/" + THINGSPEAK_WRITE_API_KEY, "0", 0, false};
mqtt_msg lastWill = {"", "", 0, false};

String ip, mqttId;
int ticks = 0;

void setup()
{
  Serial.begin(19200);

  nanoesp.init();
  nanoesp.configWifiMode(1);
  nanoesp.configWifiStation(SSID, PASSWORD);

  if (nanoesp.wifiConnected())
  {
    Serial.println(F("Wifi Connected"));
    digitalWrite(LED_WLAN, HIGH);
  }
  else
  {
    Serial.println(F("Wifi not Connected"));
  }

  //Print IP in Terminal
  nanoesp.getIpMac(ip, mqttId);
  Serial.println("My IP: " + ip);

  //ConnectionID, Brooker, Port, Clean Session, Keep alive Time (def. 120s), Pointer to lastWill MSG, Username, Password (MQTT API KEY)
  if (mqtt.connect(0, "mqtt.thingspeak.com", 1883, mqttId, true, 120, &lastWill, "randomname", THINGSPEAK_MQTT_API_KEY))
  {
    Serial.println(F("Connected to Broker"));
  }
  else
  {
    Serial.println(F("error connecting"));
    nanoesp.serialDebug();
  }
}

void loop()
{
  int id, len;

  if (nanoesp.recvData(id, len))
  {
    mqtt.recvMQTT(id, len); //auto hanlder mit Funktionen
  }
  mqtt.stayConnected(0);

  sendValues();
  delay(100);
}

void sendValues()
{
  if (ticks > 0)
  {
    ticks--;
  }
  else
  {
    newValues.value = String("field1=" + String(getTemp(), DEC) + "&field2=" + String(getLight(), DEC));
    Serial.println("Update Values: " + newValues.value);
    mqtt.publish(0, &newValues);

    ticks = 150;
  }
}

//--------------------------------Sensors----------------------------------------------------
double getTemp()
{
  //Source: http://playground.arduino.cc/ComponentLib/Thermistor2
  double Temp;
  int RawADC = analogRead(TEMP);
  Temp = log(10000.0 * ((1024.0 / RawADC - 1)));
  //         =log(10000.0/(1024.0/RawADC-1)) // for pull-up configuration
  Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp)) * Temp);
  Temp = Temp - 273.15; // Convert Kelvin to Celcius
  // Temp = (Temp * 9.0)/ 4.7 + 32.0; // Convert Celcius to Fahrenheit
  return Temp;
}

int getLight()
{
  return analogRead(LIGHT);
}
