/*
  Thingspeak and MQTT - Twitter Alarm
  Change SSID and PASSWORD.
  Change Thingsspeak Access Data

  Change SoftwareSerial.h -> #define _SS_MAX_RX_BUFF 128 // RX buffer size 
*/

#include <NanoESP.h>
#include <NanoESP_MQTT.h>
#include <SoftwareSerial.h>
#include <LiquidCrystal.h>

#define SSID "[Your SSID]"
#define PASSWORD "[Your Password]"

#define ALARM_TEMP 23
#define THINGSPEAK_CHANNEL_ID "842535"
#define THINGSPEAK_WRITE_API_KEY "J88WXG8DF1JID9TB"
#define THINGSPEAK_READ_API_KEY "JPOKORV4RVTRKTR0"
#define THINGSPEAK_MQTT_API_KEY "YNH3T1JOIZ9MM9AI"

#define LED_WLAN 13

#define TEMP A7
#define LIGHT A6
#define PIEZO 8

NanoESP nanoesp = NanoESP();
NanoESP_MQTT mqtt = NanoESP_MQTT(nanoesp);

LiquidCrystal lcd(A0, A1, A2, A3, A4, A5);

//topic,         value,    QOS, retain
mqtt_msg newValues = {"channels/" + String(THINGSPEAK_CHANNEL_ID) + "/publish/" + THINGSPEAK_WRITE_API_KEY, "0", 0, false};
mqtt_msg lastWill = {"", "", 0, false};

String ip, mqttId;
int ticks = 0;
int ticksAlarm = 0;
bool alarmState = false;

void setup()
{
  Serial.begin(19200);
  if (_SS_MAX_RX_BUFF < 128)
    Serial.println(F("ERROR: Increase Software Serial buffer size!"));
  lcd.begin(16, 2);
  lcd.print(F("ThingSpeak Alarm"));

  nanoesp.init();
  nanoesp.configWifiMode(1);
  nanoesp.configWifiStation(SSID, PASSWORD);

  if (nanoesp.wifiConnected())
  {
    Serial.println(F("Wifi Connected"));
    digitalWrite(LED_WLAN, HIGH);

    lcd.setCursor(0, 1);
    lcd.print(F("WLAN CONNECTED"));
  }
  else
  {
    Serial.println(F("Wifi not Connected"));
    lcd.setCursor(0, 1);
    lcd.print(F("WLAN ERROR"));
  }

  //Print IP in Terminal
  nanoesp.getIpMac(ip, mqttId);
  Serial.println("My IP: " + ip);

  //ConnectionID, Brooker, Port, Clean Session, Keep alive Time (def. 120s), Pointer to lastWill MSG, Username, Password (MQTT API KEY)
  if (mqtt.connect(0, "mqtt.thingspeak.com", 1883, mqttId, true, 120, &lastWill, "randomname", THINGSPEAK_MQTT_API_KEY))
  {
    Serial.println(F("Connected to Broker"));
    if (mqtt.subscribe(0, "channels/" + String(THINGSPEAK_CHANNEL_ID) + "/subscribe/fields/field1/" + String(THINGSPEAK_READ_API_KEY), 0, temp_data_update))
      Serial.println(F("Subscribed to temperature update"));
  }
  else
  {
    Serial.println(F("error connecting"));
    nanoesp.serialDebug();
  }
}

void loop()
{
  int id, len;
  String value, topic;

  if (nanoesp.recvData(id, len))
  {
    if (nanoesp.overflow())
      Serial.println(F("SoSe buffer overflow!"));
    mqtt.recvMQTT(id, len);
  }
  if (alarmState)
    sireneTone();
  mqtt.stayConnected(0);
  sendValues();
  delay(100);
}

void sendValues()
{

  float temp = roundf(getTemp() * 100.0) / 100.0; // will round to 2 decimal places i.e. 0.00
  int light = getLight();

  if (ticks % 10 == 0)
  {
    lcd.setCursor(0, 1);
    lcd.print(("T: " + String(temp) + " L: " + String(light)));
  }
  if (ticks > 0)
  {
    ticks--;
  }
  else
  {
    newValues.value = String("field1=" + String(temp) + "&field2=" + String(light));
    Serial.println("Update Values: " + newValues.value);
    mqtt.publish(0, &newValues);

    ticks = 150;
  }
}

//--------------------------------Sensors----------------------------------------------------
double getTemp()
{
  //Source: http://playground.arduino.cc/ComponentLib/Thermistor2
  double Temp;
  int RawADC = analogRead(TEMP);
  Temp = log(10000.0 * ((1024.0 / RawADC - 1)));
  //         =log(10000.0/(1024.0/RawADC-1)) // for pull-up configuration
  Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp)) * Temp);
  Temp = Temp - 273.15; // Convert Kelvin to Celcius
  // Temp = (Temp * 9.0)/ 4.7 + 32.0; // Convert Celcius to Fahrenheit
  return Temp;
}

int getLight()
{
  return analogRead(LIGHT);
}

void temp_data_update(String value)
{
  Serial.println("TEMP: " + value);
  float temp = value.toFloat();
  if (temp > ALARM_TEMP)
  {
    alarm(true);
  }
  else
  {
    alarm(false);
  }
}

void alarm(boolean alarm)
{
  if (alarm && !alarmState)
  {
    alarmState = true;
    Serial.println(F("ALARM!"));
    lcd.setCursor(0, 0);
    lcd.print(F(" - Alarm!-      "));
  }
  else if (!alarm && alarmState)
  {
    alarmState = false;
    Serial.println(F("Alarm deactivated"));
    lcd.setCursor(0, 0);
    lcd.print(F("Thingspeak Alarm"));
  }
}

void sireneTone()
{
  if (ticksAlarm < 5)
    tone(PIEZO, 400, 500);
  else
    tone(PIEZO, 800, 500);
  ticksAlarm++;
  if (ticksAlarm > 10)
    ticksAlarm = 0;
}